/*===========================================================================*/
/*									     */
/* 	Module "io_field.c"						     */
/*	===================						     */
/*	contient les procedures d'entree/sortie pour les champs de vecteurs  */
/*									     */
/*===========================================================================*/
/*									     */
/*   ajout J.M. ODOBEZ : Procedures d'allocation d'un champ de reels         */
/*									     */
/*****************************************************************************/

#include <stdio.h>
#include <sys/file.h>
#include <fcntl.h>
#include "io_field.h"

/*===========================================================================*/
/* procedure d'ecriture d'un champ code en float dans un fichier code en char*/
/*
 * name_field : nom du fichier
 * ima_u, ima_v : champs de vecteurs (float)
 * nbcol, nblig : dimension du champ (short)
*/
/*--------------------------------------------------------------------------*/
void write_field(char *name_field,float *ima_u, float *ima_v,short nbcol,
		 short nblig,short *error)
{
  	short		CL, NL, cl, nl;
	register int	i, j;
	int 		fichier;
	char 		entete[4];
	char 		*buffer, c, *pt;
	unsigned char 	c1;
  	float		*field_u, *field_v;
  	float		*ptamp, *porig;
	float		*pf;
	int		libere;

	*error = 0;

	field_u = (float *) calloc((unsigned int)(nbcol * nblig), sizeof(float));
	field_v = (float *) calloc((unsigned int)(nbcol * nblig), sizeof(float));

	porig = ima_u; ptamp = field_u;
	for (i=0; i<nblig*nbcol; i++) *ptamp++ = *porig++;
	porig = ima_v; ptamp = field_v;
	for (i=0; i<nblig*nbcol; i++) *ptamp++ = *porig++;

	fichier = creat(name_field,0644);
	if (fichier == -1) {
	  perror("fichier vecteurs");
	  *error = 1;
	  return;
	}

	if ((buffer = (char *) calloc((unsigned int)(2*nbcol*nblig),sizeof(char))) == NULL) {
	  perror("allocation");
	  *error = 2;
	  return;
	}

	CL = (short )(nbcol / 256);
	cl =  (short)(nbcol - 256*CL);
	NL = (short)(nblig / 256);
	nl =  (short)(nblig - 256*NL);

	entete[0] = (char)(NL);
	entete[1] = (char)(nl);
	entete[2] = (char)(CL);
	entete[3] = (char)(cl);

	write(fichier,entete, 4*sizeof(char));

	pt = buffer; pf = field_u;
	for(i=0 ; i<nblig ; i++)  for(j=0 ; j<nbcol ; j++) {
	  (*pf) += (*pf >= 0.0) ? 1 : -1;
	  c = (char) *pf;
	  if ( *pf >= 0) c1 = ((*pf -  c) * 200.0);
	  else c1 = ((c - *pf) * 200.0);
	  *pt++ = c;
	  *pt++ = c1;
	  pf ++;
	}
	write(fichier,buffer, 2*nbcol*nblig*sizeof(char));

	pt = buffer; pf = field_v;
	for(i=0 ; i<nblig ; i++)  for(j=0 ; j<nbcol ; j++) {
	  (*pf) += (*pf >= 0.0) ? 1 : -1;
	  c = (char) *pf;
	  if ( *pf >= 0) c1 = ((*pf -  c) * 200.0);
	  else c1 = ((c - *pf) * 200.0);
	  *pt++ = c;
	  *pt++ = c1;
	  pf ++;
	}
	write(fichier,buffer,2*nbcol*nblig*sizeof(char));

	free((char *)buffer);	/* 	if(!libere) printf("\nwrite_field erreur free()"); */
	free((char *)field_u);/* 	if(!libere) printf("\nwrite_field erreur free()"); */
	free((char *)field_v);	/* if(!libere) printf("\nwrite_field erreur free()"); */


	close(fichier);
}

/*==========================================================================*/
/* procedure de lecture de l'entete d'un fichier de vecteurs                */
/* routine a utiliser pour connaitre les dimensions du champ		    */
/*
entrees :
	- name_field : nom du champ

sorties :
	- nbcol, nblig : nombre de col., nombre de lignes du champ
*/
/*--------------------------------------------------------------------------*/
void read_entete_field(char	name_field[80],short *nbcol,short *nblig, short *error)
/*   char		name_field[80]; */
/*   short		*nbcol, *nblig; */
/*   short		*error; */
{
  	unsigned short		CL, NL, cl, nl;
	int 		fichier;
	char 		entete[4];

	*error = 0;

	fichier = open(name_field,O_RDONLY);
	if (fichier == -1){
	  perror("fichier vect.");
	  *error = 1;
	  return;
	}

	read(fichier,entete, 4*sizeof(char));

	NL = (short)entete[0];
	nl = (short)entete[1];
	CL = (short)entete[2];
	cl = (short)entete[3];

	*nblig = 256*NL + nl;
	*nbcol = 256*CL + cl;

	close(fichier);
}

/*===========================================================================*/
/* procedure de lecture d'un fichier code en char dans un champ code en float*/
/*
entrees :
	name_field : nom du fichier
	nbcol, nblig : dimension du champ (short)
sorties
	ima_u, ima_v : champs de vecteurs (float)
*/
/*--------------------------------------------------------------------------*/
void read_field(char	name_field[80], float *field_u, float *field_v,short col,
	   short lig,short *error)
/*   char		name_field[80]; */
/*   float		*field_u, *field_v; */
/*   short		col, lig; */
/*   short		*error; */
{
	int 		fichier;
	float 		*ptf1, *ptf2;
	char 		*buffer, c, *pt;
	char 		entete[4];
	unsigned char 	c1;
	*error = 0;

	fichier = open(name_field,O_RDONLY);
	if (fichier == -1){
	  perror("fichier");
	  *error = 1;
	  return;
	}

	read(fichier,entete, 4*sizeof(char));

	if ((buffer = (char *) calloc((unsigned int)(2*lig*col), sizeof(char))) == NULL) {
	  perror("allocation");
	  *error = 2;
	  return;
	}

	read(fichier,buffer,2*lig*col*sizeof(char));
	pt = buffer;
	ptf1 = field_u;
	ptf2 = field_u + (lig * col);
	while ( ptf1 < ptf2) {
	  c = *pt++;
	  c1=  *pt++;
	  *ptf1 = (float) c ;
	  if ( *ptf1 < 0 )  *ptf1 -= ((float) (unsigned) c1 / 200.0) - 1;
	  else  *ptf1 += ((float) (unsigned) c1 / 200.0) - 1;
	  *ptf1++;
	}

	read(fichier,buffer,2*lig*col*sizeof(char));
	close(fichier);
	pt = buffer;
	ptf1 = field_v;
	ptf2 = field_v + (lig * col);
	while ( ptf1 < ptf2) {
	  c = *pt++;
	  c1=  *pt++;

	  *ptf1 = (float) c ;
	  if ( *ptf1 < 0 )  *ptf1 -= ((float) (unsigned) c1 / 200.0) - 1;
	  else  *ptf1 += ((float) (unsigned) c1 / 200.0) - 1;
	  *ptf1++;
	}

	free(buffer);
}

